@@ -44,6 +44,7 @@ extern afs_rwlock_t afs_xcbhash;
 extern afs_int32 afs_mariner, afs_marinerHost;
 extern struct srvAddr *afs_srvAddrs[NSERVERS];
 extern int afs_resourceinit_flag;
+extern afs_int32 cryptall;
 
 static int PBogus(), PSetAcl(), PGetAcl(), PSetTokens(), PGetVolumeStatus();
 static int PSetVolumeStatus(), PFlush(), PNewStatMount(), PGetTokens(), PUnlog();
@@ -59,6 +60,7 @@ static int PSetSPrefs(), PGetSPrefs(), PGag(), PTwiddleRx();
 static int PSetSPrefs33(), PStoreBehind(), PGCPAGs();
 static int PGetCPrefs(), PSetCPrefs(); /* client network addresses */
 static int PGetInitParams(), PFlushMount(), PRxStatProc(), PRxStatPeer();
+static int PGetRxkcrypt(), PSetRxkcrypt();
 int PExportAfs();
 
 static int HandleClientContext(struct afs_ioctl *ablob, int *com, struct AFS_UCRED **acred, struct AFS_UCRED *credp);
@@ -121,6 +123,17 @@ static int (*(pioctlSw[]))() = {
   PFlushMount,			/* 52 - flush mount symlink data */
   PRxStatProc,			/* 53 - control process RX statistics */
   PRxStatPeer,			/* 54 - control peer RX statistics */
+  PGetRxkcrypt,			/* 55 -- Get rxkad encryption flag */
+  PSetRxkcrypt,			/* 56 -- Set rxkad encryption flag */
+  PNoop,			/* 57 -- arla: set file prio */
+  PNoop,			/* 58 -- arla: fallback getfh */
+  PNoop,			/* 59 -- arla: fallback fhopen */
+  PNoop,			/* 60 -- arla: controls xfsdebug */
+  PNoop,			/* 61 -- arla: controls arla debug */
+  PNoop,			/* 62 -- arla: debug interface */
+  PNoop,			/* 63 -- arla: print xfs status */
+  PNoop,			/* 64 -- arla: force cache check */
+  PNoop,			/* 65 -- arla: break callback */
 };
 
 #define PSetClientContext 99	/*  Special pioctl to setup caller's creds  */
@@ -180,6 +193,21 @@ copyin_afs_ioctl(caddr_t cmarg, struct afs_ioctl *dst)
 	}
 #endif /* defined(AFS_SGI_ENV) && (_MIPS_SZLONG==64) */
 
+#if defined(AFS_LINUX_64BIT_KERNEL)
+	struct afs_ioctl32 dst32;
+
+#ifdef AFS_SPARC64_LINUX20_ENV
+	if (current->tss.flags & SPARC_FLAG_32BIT) {
+#else
+#error Not done for this linux type
+#endif
+		AFS_COPYIN(cmarg, (caddr_t) &dst32, sizeof dst32, code);
+		if (!code)
+			afs_ioctl32_to_afs_ioctl(&dst32, dst);
+		return code;
+	}
+#endif /* defined(AFS_LINUX_64BIT_KERNEL) */
+
 	AFS_COPYIN(cmarg, (caddr_t) dst, sizeof *dst, code);
 	return code;
 }
@@ -792,7 +820,7 @@ afs_syscall_pioctl(path, com, cmarg, follow)
 #ifdef AFS_LINUX22_ENV
 	code = gop_lookupname(path, AFS_UIOUSER, follow,  (struct vnode **) 0, &dp);
 	if (!code)
-	    vp = (struct vcache *)dp->d_inode;
+	    vp = (struct vnode *)dp->d_inode;
 #else
 	code = gop_lookupname(path, AFS_UIOUSER, follow,  (struct vnode **) 0, &vp);
 #endif /* AFS_LINUX22_ENV */
@@ -2753,7 +2781,7 @@ static int afs_setsprefs(sp, num, vlonly)
       /* Note that it doesn't have a cell yet...     */
       if (!matches) {
 	 afs_uint32 temp = sp->host.s_addr;
-	 srvr = afs_GetServer(&temp, 1, NULL, (vlonly ? AFS_VLPORT : AFS_FSPORT), 
+	 srvr = afs_GetServer(&temp, 1, 0, (vlonly ? AFS_VLPORT : AFS_FSPORT), 
 			      WRITE_LOCK, (afsUUID *)0,0);
 	 srvr->addr->sa_iprank = sp->rank + afs_randomMod15();
 	 afs_PutServer(srvr, WRITE_LOCK);
@@ -3106,6 +3134,46 @@ static cred_t *crget(void)
     return cr;
 }
 #endif
+
+static int
+PGetRxkcrypt(avc, afun, areq, ain, aout, ainSize, aoutSize, acred)
+struct vcache *avc;
+int afun;
+struct vrequest *areq;
+char *ain, *aout;
+afs_int32 ainSize;
+afs_int32 *aoutSize;
+struct AFS_UCRED *acred;
+{
+    bcopy((char *)&cryptall, aout, sizeof(afs_int32));
+    *aoutSize=sizeof(afs_int32);
+    return 0;
+}
+
+static int
+PSetRxkcrypt(avc, afun, areq, ain, aout, ainSize, aoutSize, acred)
+struct vcache *avc;
+int afun;
+struct vrequest *areq;
+char *ain, *aout;
+afs_int32 ainSize;
+afs_int32 *aoutSize;
+struct AFS_UCRED *acred;
+{
+    afs_int32 tmpval;
+
+    if (!afs_osi_suser(acred))
+      return EPERM;
+    if (ainSize != sizeof(afs_int32) || ain == NULL)
+      return EINVAL;
+    bcopy(ain, (char *)&tmpval, sizeof(afs_int32));
+    /* if new mappings added later this will need to be changed */
+    if (tmpval != 0 && tmpval != 1)
+      return EINVAL;
+    cryptall = tmpval;
+    return 0;
+}
+
 /*
  * Create new credentials to correspond to a remote user with given
  * <hostaddr, uid, g0, g1>.  This allows a server running as root to
@@ -3318,8 +3386,10 @@ afs_int32 *aoutSize;
 
 	if ( ainSize < sizeof(struct setspref) )
 		return EINVAL;
+#if 0	/* num_servers is unsigned */
 	if ( sin->num_servers < 0 )
 		return EINVAL;
+#endif
 	if ( sin->num_servers > AFS_MAX_INTERFACE_ADDR)
 		return ENOMEM;
 
